#include "..\Plugin API\ExtAPI.h"
#include "zCalcHF.h"
#include "..\Plugin API\zVar.h"
#include "..\Plugin API\zStr.h"
#include "..\Plugin API\zMap.h"
#include "..\Plugin API\zFunc.h"

extern CExtAPI theAPI;

bool zCalcHF_Inflate(ZMAP pHF, ZMAP pDM, long nLevels) {
	CzFunc f;
	if(!f.GetFunc("calc.HF.Inflate")) {
		return false;
	}

	long ArgID = 0;
	if(!theAPI.var_SetVarRef(f.m_args.GetItem(ArgID++), pHF))
		return false;
	if(!theAPI.var_SetVarRef(f.m_args.GetItem(ArgID++), pDM))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_int, &nLevels))
		return false;

	CzVar rval;
	rval.Attach(f.Execute());

	bool b = false;
	if(!rval.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}

bool zCalcHF_InflateMosaic(ZMAP pMapSrc, ZMAP pMapDest, const char* lpFileName, const char* lpMapName, long TileSize, ZFORMAT pFormat, ZMAP pDM, long nLevels) {

	CzFunc f;
	if(!f.GetFunc("calc.HF.InflateMosaic")) {
		theAPI.ReportError("zCalcHF_InflateMosaic error:\r\n - cannot get 'calc.HF.InflateMosaic' function");
		return false;
	}

	CString TempStr;

	long ArgID = 0;
	if(!theAPI.var_SetVarRef(f.m_args.GetItem(ArgID++), pMapSrc)) {
		TempStr.Format("zCalcHF_InflateMosaic error:\r\n - cannot set argument %d", ArgID-1);
		theAPI.ReportError(TempStr);
		return false;
	}
	if(!theAPI.var_SetVarRef(f.m_args.GetItem(ArgID++), pMapDest)) {
		TempStr.Format("zCalcHF_InflateMosaic error:\r\n - cannot set argument %d", ArgID-1);
		theAPI.ReportError(TempStr);
		return false;
	}
	if(!theAPI.str_SetText(f.m_args.GetItem(ArgID++), lpFileName)) {
		TempStr.Format("zCalcHF_InflateMosaic error:\r\n - cannot set argument %d", ArgID-1);
		theAPI.ReportError(TempStr);
		return false;
	}
	if(!theAPI.str_SetText(f.m_args.GetItem(ArgID++), lpMapName)) {
		TempStr.Format("zCalcHF_InflateMosaic error:\r\n - cannot set argument %d", ArgID-1);
		theAPI.ReportError(TempStr);
		return false;
	}
	if(!f.m_args.SetItemValue(ArgID++, VarID_int, &TileSize)) {
		TempStr.Format("zCalcHF_InflateMosaic error:\r\n - cannot set argument %d", ArgID-1);
		theAPI.ReportError(TempStr);
		return false;
	}
	if(!theAPI.var_SetVarRef(f.m_args.GetItem(ArgID++), pFormat)) {
		TempStr.Format("zCalcHF_InflateMosaic error:\r\n - cannot set argument %d", ArgID-1);
		theAPI.ReportError(TempStr);
		return false;
	}
	if(!theAPI.var_SetVarRef(f.m_args.GetItem(ArgID++), pDM)) {
		TempStr.Format("zCalcHF_InflateMosaic error:\r\n - cannot set argument %d", ArgID-1);
		theAPI.ReportError(TempStr);
		return false;
	}
	if(!f.m_args.SetItemValue(ArgID++, VarID_int, &nLevels)) {
		TempStr.Format("zCalcHF_InflateMosaic error:\r\n - cannot set argument %d", ArgID-1);
		theAPI.ReportError(TempStr);
		return false;
	}

	if(!f.Execute2(NULL, NULL)) {
		theAPI.ReportError("zCalcHF_InflateMosaic error:\r\n - call to 'calc.HF.InflateMosaic' failed");
		return false;
	}

	bool b = false;
	if(!f.m_rval.GetValue(VarID_bool, &b)) {
		theAPI.ReportError("zCalcHF_InflateMosaic error:\r\n - cannot get function return value");
		TempStr.Format("Type = %d\r\n", f.m_rval.GetTypeID());
		theAPI.ReportError(TempStr);
		return false;
	}

	return b;
}

bool zCalcHF_PeakPass(ZMAP pHF, ZMAP pDM) {
	
	CzFunc f;
	if(!f.GetFunc("calc.HF.PeakPass")) {
		return false;
	}

	long ArgID = 0;
	if(!theAPI.var_SetVarRef(f.m_args.GetItem(ArgID++), pHF))
		return false;
	if(!theAPI.var_SetVarRef(f.m_args.GetItem(ArgID++), pDM))
		return false;
	
	CzVar rval;
	rval.Attach(f.Execute());

	bool b = false;
	if(!rval.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}
		
bool zCalcHF_MountainPass(ZMAP pHF, ZMAP pDM) {
	
	CzFunc f;
	if(!f.GetFunc("calc.HF.MountainPass")) {
		return false;
	}

	long ArgID = 0;
	if(!theAPI.var_SetVarRef(f.m_args.GetItem(ArgID++), pHF))
		return false;
	if(!theAPI.var_SetVarRef(f.m_args.GetItem(ArgID++), pDM))
		return false;
	
	CzVar rval;
	rval.Attach(f.Execute());

	bool b = false;
	if(!rval.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}

bool zCalcHF_VolcanoPass(ZMAP pHF, ZMAP pDM) {
	
	CzFunc f;
	if(!f.GetFunc("calc.HF.VolcanoPass")) {
		return false;
	}

	long ArgID = 0;
	if(!theAPI.var_SetVarRef(f.m_args.GetItem(ArgID++), pHF))
		return false;
	if(!theAPI.var_SetVarRef(f.m_args.GetItem(ArgID++), pDM))
		return false;
	
	CzVar rval;
	rval.Attach(f.Execute());

	bool b = false;
	if(!rval.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}

bool zCalcHF_PlateauPass(ZMAP pHF, ZMAP pDM) {
	
	CzFunc f;
	if(!f.GetFunc("calc.HF.PlateauPass")) {
		return false;
	}

	long ArgID = 0;
	if(!theAPI.var_SetVarRef(f.m_args.GetItem(ArgID++), pHF))
		return false;
	if(!theAPI.var_SetVarRef(f.m_args.GetItem(ArgID++), pDM))
		return false;
	
	CzVar rval;
	rval.Attach(f.Execute());

	bool b = false;
	if(!rval.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}

bool zCalcHF_TerracePass(ZMAP pHF, ZMAP pDM) {
	
	CzFunc f;
	if(!f.GetFunc("calc.HF.TerracePass")) {
		return false;
	}

	long ArgID = 0;
	if(!theAPI.var_SetVarRef(f.m_args.GetItem(ArgID++), pHF))
		return false;
	if(!theAPI.var_SetVarRef(f.m_args.GetItem(ArgID++), pDM))
		return false;
	
	CzVar rval;
	rval.Attach(f.Execute());

	bool b = false;
	if(!rval.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}

bool zCalcHF_FileOverlayPass(ZMAP pHF, ZMAP pDM) {
	
	CzFunc f;
	if(!f.GetFunc("calc.HF.FileOverlayPass")) {
		return false;
	}

	long ArgID = 0;
	if(!theAPI.var_SetVarRef(f.m_args.GetItem(ArgID++), pHF))
		return false;
	if(!theAPI.var_SetVarRef(f.m_args.GetItem(ArgID++), pDM))
		return false;
	
	CzVar rval;
	rval.Attach(f.Execute());

	bool b = false;
	if(!rval.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}

bool zCalcHF_ThermalPass(ZMAP pHF, ZMAP pDM, int nLevelMultiplier, int LevelOffset, int MaxLength, float S, float GradThreshold) {
	
	CzFunc f;
	if(!f.GetFunc("calc.HF.ThermalPass")) {
		return false;
	}

	long ArgID = 0;
	if(!theAPI.var_SetVarRef(f.m_args.GetItem(ArgID++), pHF))
		return false;
	if(!theAPI.var_SetVarRef(f.m_args.GetItem(ArgID++), pDM))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_int, &nLevelMultiplier))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_int, &LevelOffset))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_int, &MaxLength))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_float, &S))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_float, &GradThreshold))
		return false;
	
	CzVar rval;
	rval.Attach(f.Execute());

	bool b = false;
	if(!rval.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}

bool zCalcHF_ChannelPass(ZMAP pHF, ZMAP pDM, int nLevelMultiplier, int LevelOffset, int MaxLength, float DeltaHF, float MemDecay, bool BlurDir) {
	
	CzFunc f;
	if(!f.GetFunc("calc.HF.ChannelPass")) {
		return false;
	}

	long ArgID = 0;
	if(!theAPI.var_SetVarRef(f.m_args.GetItem(ArgID++), pHF))
		return false;
	if(!theAPI.var_SetVarRef(f.m_args.GetItem(ArgID++), pDM))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_int, &nLevelMultiplier))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_int, &LevelOffset))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_int, &MaxLength))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_float, &DeltaHF))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_float, &MemDecay))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_bool, &BlurDir))
		return false;
	
	CzVar rval;
	rval.Attach(f.Execute());

	bool b = false;
	if(!rval.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}

bool zCalcHF_CurvFilter(ZMAP pHF, long nLevels, float Threshold, bool ShowProgress) {
	
	CzFunc f;
	if(!f.GetFunc("calc.HF.CurvFilter")) {
		return false;
	}

	long ArgID = 0;
	if(!theAPI.var_SetVarRef(f.m_args.GetItem(ArgID++), pHF))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_int, &nLevels))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_float, &Threshold))
		return false;
	if(!f.m_args.SetItemValue(ArgID++, VarID_bool, &ShowProgress))
		return false;
	LPVOID hMaskMap = NULL; // note, we won't set the mask map in this version
	if(!f.m_args.SetItemValue(ArgID++, VarID_LPVOID, &hMaskMap))
		return false;

	CzVar rval;
	rval.Attach(f.Execute());

	bool b = false;
	if(!rval.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}